+++
title = "Open Source and Mental Health"
author = "jackpot51"
date = "2021-06-12"
+++

A dear friend of mine and prolific Redox OS contributor, jD91mZM2, passed away
in March of 2021, at the age of 18. He was involved in the Redox OS Summer of
Code for 2018, 2019, and 2020. He was instrumental in developing all aspects of
Redox OS, from the kernel, to relibc, to porting programs. His work was
detailed in his own words in a number of news posts authored by jD91mZM2 at
https://www.redox-os.org/news/.

This post is likely to be dark, deep, heavy, raw, and unedited. If you are
having issues of your own, as many of us are, feel free to contact me at:
https://twitter.com/jeremy_soller.
I do not recommend reading the rest of this post in this case. My conclusion is
that open source has significant work to do to become sustainable, and much of
this is caring for the health of the community and its members.

## Reflection

Yesterday, I was messaged by another contributor regarding the length of time
jD91mZM2 was offline and the fact he was not responding to emails. I reached
out via what I had, to no avail. I revealed his true name to the other
contributor, who then found his obituary. We verified that his name, location,
and date of birth matched. While no cause of death was listed, I believe the
evidence we found points to suicide after a mental health episode.

After learning all of this, I was shocked. How could such a prolific
contributor, not just to Redox but to many projects, feel that death was
preferable to life? This was a person who was boundlessly competent, and who
seemed until recently to have a good handle on their life. But the longer I
have lived, the more I realize how much of an illusion this can be, and how
rapidly things can deteriorate.

I last communicated with jD91mZM2 in February, one month prior to his death.
This communication was purely technical, regarding the aarch64 port of the
Redox kernel. I can't help but think, that perhaps this was a factor in his
decision to choose death.

In open source, we often emphasize the importance of good code. After all, the
deliverable of every open source project, is the source, right? We often forget
that good code is written by good people, and that retaining those people and
keeping them happy should be the primary concern of any project maintainer.

There are lots of aspects to mental health episodes. On the one hand, there is
usually a genetic component to mental illness. On the other, these genetic
precursors usually require both chronic and acute environmental triggers.
These chronic triggers can be a long-lasting poor home or work environment, and
lead to manifestation of the mental illness itself. The acute triggers could
be, for example, an argument with someone, leading to a mental illness episode.
These episodes can be serious enough to overcome the extreme instinct to
survive, leading to suicide.

In this way, suicide is not a display of weakness. In fact, it is a display of
extreme conviction and strength. Even with the backdrop of mental illness,
there are parts of the brain that are usually unaffected. These parts are so
ancient in development, we have little conscious control of them. To attempt
suicide requires overcoming conscious desires to survive. To succeed, is to
overcome extreme subconscious desires. This means that, for suicide, often the
smartest and most capable people are able to succeed.

This anti-selection of capable people is a terrifying epidemic. Humanity in
general is in desperate need of artificial solutions to long-standing problems.
Take climate change, for instance. Out of the 800,000 people each year who
committed suicide, on average perhaps more capable than the rest of us, what if
a few would have been instrumental in developing fusion power?

And yet, we as a society have taken the position that these events are an
unstoppable force. That the factors leading to suicide are internal, not
external. I refuse to believe this, on principle. We must search out causes and
mitigate them, for every problem, even if it ends up being impossible.

And so, I am forced to look into my own actions, to see what could have been
done differently. To see if I could have saved a life, and to see what lives
I can save in the future.

## Open Source and Mental Health

There are aspects of open source that seem to attract the strangest of
humanity, myself included. The insistence that all things be inspectable, is
perhaps driven by obsessive compulsive behavior. And those prone to such
behavior, often inherit it from other disorders. ADHD, autism, bipolar
disorder, depression, and other disorders are incredibly common among open
source contributors.

As a result, there is also an obvious lack of soft skills among open source
communities. This has the clear effect of splitting communities and isolating
open source itself from the "normal" world. Fortunately for us, open source
ended up being a profitable business to be in. This injection of capital has
lead to a significant diversification of talent in open source.

This has, however, come at a cost. Those who do not fit into the new business
interests of open source projects are often cast aside. Projects are further
divided into forks on forks, as disagreements with the original maintainers
lead to insurmountable changes in the projects themselves. I myself have
participated in all sides of this.

Lacking in most of this has been an investigation into the human cost. To look
into the numerous mental health episodes among open source contributors, and
to try to identify some common elements. Sometimes these episodes lead to new
projects, sometimes to burn out contributors who then leave open source, and
sometimes to suicide.

We have to recognize our role in creating both the chronic stressors leading to
mental illness, as well as the acute stressors leading to dangerous episodes.

## My Own Journey

I am not immune to mental illness. I often get messages to the effect of, "You
seem to have your stuff under control, how do you do it?" The brutal truth is,
I doubt any of us actually do. And what we define as "under control" may be
very different. Having successful projects is not the same as having general
happiness.

I do have to recognize I am much happier now than I have ever been. So, perhaps
compared to the average, I do have things under control. My life has been an
odyssey of therapists, psychiatrists, drugs, and isolation. It could have
easily ended as others have. I was lucky enough to find my chronic stressors,
and dutifully eliminate them.

My freshman year of college, around the same time in my life as when jD91mZM2
decided to end his own, was particularly hard. I gained nearly fifty pounds in
weight. I lived with three other roommates, two of which have also died young.
I alternated between ADHD medication, anti-depressants, and even smoking - to
find what would "fix" me. The whole time, I programmed, often ignoring my
school work to do so.

Prior to college, I had worked as an intern writing defibrillator software at
Zoll Medical. I have two patents from this work. I'll be honest, I was leaps
and bounds above others in my understanding of computers. And I'll be honest
with myself, I was leaps and bounds below others in my understanding of people,
including myself.

In that time, I developed a relationship with the Vice President of Research
and Development, who became my defacto boss. Near the end of my freshman year,
he reached out to me to ask if I wanted to continue working. I said yes.

My sophomore year was very different. I spent most of my time writing software,
and making good money. I had no interest in school. I failed a few courses,
while testing out of many of the high-level CS classes. I soon dropped out of
college to pursue software engineering full time.

This had a huge positive influence on my mental health. I lost weight. I kept
in touch with other alumni from my college, and eventually through them, met
my wife. We bought a house together. I started Redox OS. My wife and I got
married. I started working at System76. My wife and I had a beautiful daughter.
And never since my sophomore year of college have I ever even thought about my
own mental health, or had to go to therapy or use medication. All of my
stressors were gone.

One man's mental health success does not always translate into others. Along
this journey, I have created and destroyed (through neglect) hundreds of
relationships. I have to admit, that while I am happy, I have a tendency to
cause quite the opposite in others. I keep the relationships that bring me joy,
and ignore the ones that need work. And at some point, perhaps I forgot to
keep in touch with jD91mZM2 and make sure he found the same happiness I have.

## A Solution?

There is no solution, every case is different. But I feel compelled to check in
on the many people I have lost in the years, and I hope you do too. We are
still in the dark ages of understanding the human mind, and the more we check
in with each-other, the better we will do. I know I could have made a
difference, had I just done a little more. Maybe not for jD91mZM2, but for
someone feeling much the same. I will no longer value contributors by the code
they crank out. The code doesn't write itself, and the person writing the code
needs even more maintenance than the "open source" itself.
